@@ -192,7 +192,7 @@ const CollectionIconWatcher = new Lang.Class({
 
         this._docs.forEach(
             function(doc) {
-                pixbufs.push(doc.pristinePixbuf);
+                pixbufs.push(doc.origPixbuf);
             });
 
         this._pixbuf = GdPrivate.create_collection_icon(Utils.getIconSize(), pixbufs);
@@ -204,7 +204,7 @@ const CollectionIconWatcher = new Lang.Class({
     },
 
     destroy: function() {
-        for (id in this._docConnections) {
+        for (let id in this._docConnections) {
             let doc = this._docConnections[id];
             doc.disconnect(id);
         }
@@ -228,7 +228,7 @@ const DocCommon = new Lang.Class({
         this.mtime = null;
         this.resourceUrn = null;
         this.pixbuf = null;
-        this.pristinePixbuf = null;
+        this.origPixbuf = null;
         this.defaultAppName = null;
 
         this.mimeType = null;
@@ -330,15 +330,16 @@ const DocCommon = new Lang.Class({
                                                         Gtk.IconLookupFlags.FORCE_SIZE |
                                                         Gtk.IconLookupFlags.GENERIC_FALLBACK);
 
+        let pixbuf = null;
         if (iconInfo != null) {
             try {
-                this.pixbuf = iconInfo.load_icon();
+                pixbuf = iconInfo.load_icon();
             } catch (e) {
                 log('Unable to load pixbuf: ' + e.toString());
             }
         }
 
-        this.checkEffectsAndUpdateInfo();
+        this._setOrigPixbuf(pixbuf);
     },
 
     _refreshCollectionIcon: function() {
@@ -347,11 +348,7 @@ const DocCommon = new Lang.Class({
 
             this._collectionIconWatcher.connect('icon-updated', Lang.bind(this,
                 function(watcher, pixbuf) {
-                    if (!pixbuf)
-                        return;
-
-                    this.pixbuf = pixbuf;
-                    this.checkEffectsAndUpdateInfo();
+                    this._setOrigPixbuf(pixbuf);
                 }));
         } else {
             this._collectionIconWatcher.refresh();
@@ -447,10 +444,9 @@ const DocCommon = new Lang.Class({
                         true, null, Lang.bind(this,
                             function(object, res) {
                                 try {
-                                    this.pixbuf = GdkPixbuf.Pixbuf.new_from_stream_finish(res);
-
+                                    let pixbuf = GdkPixbuf.Pixbuf.new_from_stream_finish(res);
                                     this.thumbnailed = true;
-                                    this.checkEffectsAndUpdateInfo();
+                                    this._setOrigPixbuf(pixbuf);
                                 } catch (e) {
                                     this._failedThumbnailing = true;
                                 }
@@ -480,13 +476,18 @@ const DocCommon = new Lang.Class({
         return pix;
     },
 
-    checkEffectsAndUpdateInfo: function() {
-        let emblemIcons = [];
-        let pixbuf = this.pixbuf;
-        let activeItem;
+    _setOrigPixbuf: function(pixbuf) {
+        if (pixbuf) {
+            this.origPixbuf = pixbuf;
+        }
 
-        // save the pixbuf before modifications, to use in collection icons
-        this.pristinePixbuf = pixbuf;
+        this._checkEffectsAndUpdateInfo();
+    },
+
+    _checkEffectsAndUpdateInfo: function() {
+        let emblemIcons = [];
+        let emblemedPixbuf = null;
+        let activeItem;
 
         activeItem = Application.searchCategoryManager.getActiveItem();
 
@@ -496,7 +497,7 @@ const DocCommon = new Lang.Class({
             emblemIcons.push(this._createSymbolicEmblem('emblem-shared'));
 
         if (emblemIcons.length > 0) {
-            let emblemedIcon = new Gio.EmblemedIcon({ gicon: this.pixbuf });
+            let emblemedIcon = new Gio.EmblemedIcon({ gicon: this.origPixbuf });
 
             emblemIcons.forEach(
                 function(emblemIcon) {
@@ -508,25 +509,31 @@ const DocCommon = new Lang.Class({
 
             try {
                 let iconInfo = theme.lookup_by_gicon(emblemedIcon,
-                                                     Math.max(this.pixbuf.get_width(),
-                                                              this.pixbuf.get_height()),
+                                                     Math.max(this.origPixbuf.get_width(),
+                                                              this.origPixbuf.get_height()),
                                                      Gtk.IconLookupFlags.FORCE_SIZE);
 
-                pixbuf = iconInfo.load_icon();
+                emblemedPixbuf = iconInfo.load_icon();
             } catch (e) {
                 log('Unable to render the emblem: ' + e.toString());
             }
+        } else {
+            emblemedPixbuf = this.origPixbuf;
         }
 
+        let thumbnailedPixbuf = null;
+
         if (this.thumbnailed) {
             let [ slice, border ] = Utils.getThumbnailFrameBorder();
-            this.pixbuf = Gd.embed_image_in_frame(pixbuf,
+            thumbnailedPixbuf = Gd.embed_image_in_frame(emblemedPixbuf,
                 'resource:///org/gnome/documents/thumbnail-frame.png',
                 slice, border);
         } else {
-            this.pixbuf = pixbuf;
+            thumbnailedPixbuf = emblemedPixbuf;
         }
 
+        this.pixbuf = thumbnailedPixbuf;
+
         this.emit('info-updated');
     },
 
@@ -557,6 +564,12 @@ const DocCommon = new Lang.Class({
                 }
 
                 let printOp = EvView.PrintOperation.new(docModel.get_document());
+                printOp.connect('done', Lang.bind(this,
+                    function(op, res) {
+                        if (res == Gtk.PrintOperationResult.APPLY)
+                            Application.selectionController.setSelectionMode(false);
+                    }));
+
                 let printNotification = new Notifications.PrintNotification(printOp, doc);
 
                 printOp.run(toplevel);
@@ -618,8 +631,14 @@ const LocalDocument = new Lang.Class({
     },
 
     updateTypeDescription: function() {
-        if (this.mimeType)
-            this.typeDescription = Gio.content_type_get_description(this.mimeType);
+        let description = '';
+
+        if (this.collection)
+            description = _("Collection");
+        else if (this.mimeType)
+            description = Gio.content_type_get_description(this.mimeType);
+
+        this.typeDescription = description;
     },
 
     load: function(passwd, cancellable, callback) {
@@ -777,12 +796,12 @@ const GoogleDocument = new Lang.Class({
     updateTypeDescription: function() {
         let description;
 
-        if (this.rdfType.indexOf('nfo#Spreadsheet') != -1)
+        if (this.collection)
+            description = _("Collection");
+        else if (this.rdfType.indexOf('nfo#Spreadsheet') != -1)
             description = _("Spreadsheet");
         else if (this.rdfType.indexOf('nfo#Presentation') != -1)
             description = _("Presentation");
-        else if (this.rdfType.indexOf('nfo#DataContainer') != -1)
-            description = _("Collection");
         else
             description = _("Document");
 
@@ -800,6 +819,62 @@ const GoogleDocument = new Lang.Class({
     }
 });
 
+const OwncloudDocument = new Lang.Class({
+    Name: 'OwncloudDocument',
+    Extends: DocCommon,
+
+    _init: function(cursor) {
+        this._failedThumbnailing = true;
+
+        this.parent(cursor);
+
+        // overridden
+        this.sourceName = _("ownCloud");
+
+        let defaultApp = null;
+        if (this.mimeType)
+            defaultApp = Gio.app_info_get_default_for_type(this.mimeType, true);
+
+        if (defaultApp)
+            this.defaultAppName = defaultApp.get_name();
+    },
+
+    createThumbnail: function(callback) {
+        GdPrivate.queue_thumbnail_job_for_file_async(this._file, Lang.bind(this,
+            function(object, res) {
+                let thumbnailed = GdPrivate.queue_thumbnail_job_for_file_finish(res);
+                callback(thumbnailed);
+            }));
+    },
+
+    updateTypeDescription: function() {
+        let description = '';
+
+        if (this.collection)
+            description = _("Collection");
+        else if (this.mimeType)
+            description = Gio.content_type_get_description(this.mimeType);
+
+        this.typeDescription = description;
+    },
+
+    load: function(passwd, cancellable, callback) {
+        GdPrivate.pdf_loader_load_uri_async(this.uri, passwd, cancellable, Lang.bind(this,
+            function(source, res) {
+                try {
+                    let docModel = GdPrivate.pdf_loader_load_uri_finish(res);
+                    callback(this, docModel, null);
+                } catch (e) {
+                    callback(this, null, e);
+                }
+            }));
+    },
+
+    canTrash: function() {
+        return false;
+    }
+});
+
 const SkydriveDocument = new Lang.Class({
     Name: 'SkydriveDocument',
     Extends: DocCommon,
@@ -875,12 +950,12 @@ const SkydriveDocument = new Lang.Class({
     updateTypeDescription: function() {
         let description;
 
-        if (this.rdfType.indexOf('nfo#Spreadsheet') != -1)
+        if (this.collection)
+            description = _("Collection");
+        else if (this.rdfType.indexOf('nfo#Spreadsheet') != -1)
             description = _("Spreadsheet");
         else if (this.rdfType.indexOf('nfo#Presentation') != -1)
             description = _("Presentation");
-        else if (this.rdfType.indexOf('nfo#DataContainer') != -1)
-            description = _("Collection");
         else
             description = _("Document");
 
@@ -912,7 +987,7 @@ const DocumentManager = new Lang.Class({
     },
 
     _onChangesPending: function(monitor, changes) {
-        for (idx in changes) {
+        for (let idx in changes) {
             let changeEvent = changes[idx];
 
             if (changeEvent.type == ChangeMonitor.ChangeEventType.CHANGED) {
@@ -952,6 +1027,11 @@ const DocumentManager = new Lang.Class({
                 (identifier.indexOf('https://docs.google.com') != -1));
     },
 
+    _identifierIsOwncloud: function(identifier) {
+        return (identifier &&
+                (identifier.indexOf('owncloud:') != -1));
+    },
+
     _identifierIsSkydrive: function(identifier) {
         return (identifier &&
                 (identifier.indexOf('windows-live:skydrive:') != -1));
@@ -963,6 +1043,8 @@ const DocumentManager = new Lang.Class({
 
         if (this._identifierIsGoogle(identifier))
             doc = new GoogleDocument(cursor);
+        else if (this._identifierIsOwncloud(identifier))
+            doc = new OwncloudDocument(cursor);
         else if (this._identifierIsSkydrive(identifier))
             doc = new SkydriveDocument(cursor);
         else
