@@ -1,5 +1,6 @@
 /*
  * Copyright (c) 2012 Meg Ford
+ * Copyright (c) 2012, 2013 Red Hat, Inc.
  *
  * Gnome Documents is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by the
@@ -22,6 +23,7 @@
 const Gio = imports.gi.Gio;
 const GLib = imports.gi.GLib;
 const Gtk = imports.gi.Gtk;
+const Pango = imports.gi.Pango;
 const _ = imports.gettext.gettext;
 const C_ = imports.gettext.pgettext;
 
@@ -166,17 +168,31 @@ const PropertiesDialog = new Lang.Class({
             this._sourceData = new Gtk.LinkButton({ label: doc.sourceName,
                                                     uri: 'http://docs.google.com/',
                                                     halign: Gtk.Align.START });
+        } else if (doc instanceof Documents.OwncloudDocument) {
+            let source = Application.sourceManager.getItemById(doc.resourceUrn);
+            let account = source.object.get_account();
+            let presentation_identity = account.presentation_identity;
+            this._sourceData = new Gtk.LinkButton({ label: presentation_identity,
+                                                    uri: 'https://' + presentation_identity + '/',
+                                                    halign: Gtk.Align.START });
         } else if (doc instanceof Documents.SkydriveDocument) {
             this._sourceData = new Gtk.LinkButton({ label: doc.sourceName,
                                                     uri: 'https://skydrive.live.com',
                                                     halign: Gtk.Align.START });
         } else { // local document
-            let sourceLink = Gio.file_new_for_uri(doc.uri).get_parent();
-            let sourcePath = sourceLink.get_path();
+            if (doc.collection) {
+                this._sourceData = new Gtk.Label({ label: doc.sourceName,
+                                                   halign: Gtk.Align.START });
+            } else {
+                let sourceLink = Gio.file_new_for_uri(doc.uri).get_parent();
+                let sourcePath = sourceLink.get_path();
 
-            this._sourceData = new Gtk.LinkButton({ label: sourcePath,
-                                                    uri: sourceLink.get_uri(),
-                                                    halign: Gtk.Align.START });
+                this._sourceData = new Gtk.LinkButton({ label: sourcePath,
+                                                        uri: sourceLink.get_uri(),
+                                                        halign: Gtk.Align.START });
+                let label = this._sourceData.get_child();
+                label.set_ellipsize(Pango.EllipsizeMode.END);
+            }
         }
 
         grid.attach_next_to (this._sourceData, this._source, Gtk.PositionType.RIGHT, 2, 1);
